/*********************************************/
/* Exemplo de uso de macros e funes para   */
/* tratamento de excees de ponto-flutuante */
/*                                           */
/* Funes demonstradas:                     */
/*       * fegetenv()                        */
/*       * fesetenv()                        */
/*                                           */
/* Demonstra ainda o uso de #pragma:         */
/*       * STDC FENV_ACCESS                  */
/*********************************************/

#include <stdio.h>
#include <float.h>
#include <fenv.h>

/***
*
* A funo a seguir altera o ambiente de ponto
* flutuante para realizar algumas operaes, mas
* restaura o ambiente original antes de retornar
*
***/
double UmaFuncao(double x, double y)
{
      /* Esta opo fica ativada at o  */
      /* final da execuo desta funo */
#pragma STDC FENV_ACCESS ON

   double  resultado = 0.0;
   fenv_t  ambienteGuardado;

      /* Guarda o ambiente de ponto */
      /* flutuante original         */
   fegetenv(&ambienteGuardado);

      /* Instala o ambiente de  */
      /* ponto flutuante padro */
   fesetenv(FE_DFL_ENV);

      /* Executa operaes de ponto flutuante */
   /* ... */

      /* Antes de retornar, restaura */
      /* o ambiente original         */
   fesetenv(&ambienteGuardado);

   return resultado;
}

int main()
{
   double resultado;

       /* Talvez o ambiente de ponto-flutuante seja */
       /* alterado no trecho de programa a seguir   */
   /* ... */

   resultado = UmaFuncao(2.4, -0.5);

       /* Neste ponto, o ambiente de ponto-flutuante */
       /*  o mesmo de antes da chamada da funo    */
   /* ... */

   return 0;
}

